;	Main file to run from extended RAM
;
;	@com.wudsn.ide.asm.outputfileextension=.xex


	.if .not .def standalone
	.def standalone = 1
	.endif

	icl "ST-ONT-Equates.asm"

vbxe_board	equ $80
bank_number	equ $81
block_number	equ $82

	org $2000
	m_version

	.proc fader
	ldy #16			;Fade out colors registers
fade_loop
	ldx #8
color_loop
	lda $2c0,x
	and #$0f
	bne not_black
	sta PCOLR0,x
	beq next_color
not_black
	dec PCOLR0,x
next_color
	dex
	bne color_loop
	lda RTCLOK+2
	clc
	adc #3
wait	cmp RTCLOK+2
	bne wait
	dey
	bne fade_loop
	sty SDMCTL	;Disable screen DMA
	lda RTCLOK+2
wait2	cmp RTCLOK+2	;Wait for screen to be disabled before writing to $BCxx
	beq wait2
	rts
	.endp
	
	ini fader

	org $8000

	.proc loader

	.proc init
	
	.macro m_error
	lda #<text
	ldx #>text
	jmp display_error 
text	.sb ":1"
	.endm

	jsr detect_portb_banks
	mva #$22 SDMCTL
	cpy #scene_16k_banks
	bcs banks_detected
	.if scene_16k_banks >= 32
	m_error " 1 MB RAM REQUIRED. "
	.endif.
	.if scene_16k_banks >= 16
	m_error "512 KB RAM REQUIRED."
	.endif.
	.if scene_16k_banks >= 8
	m_error "256 KB RAM REQUIRED."
	.endif.
	.if scene_16k_banks >= 4
	m_error "128 KB RAM REQUIRED."
	.endif.
banks_detected

	jsr detect_vbxe
	cpx #1
	bcs vbxe_dectected
	m_error "VBXE BOARD REQUIRED."
vbxe_dectected

	adw savmsc #40*11+10 print_block.sm_adr
	mva #0 bank_number
	jsr set_bank

	mva #8 COLOR1
	mva #0 COLOR2
	mva #1 CRSINH			;Cursor off
	jsr clear_screen
	mva #0 block_number
	jmp print_block

	.proc clear_screen
	lda $e407			;PUTBYTE
	pha
	lda $e406
	pha
	lda #125			;Clear screen
	rts
	.endp

	.proc detect_portb_banks
	sei
	inc CRITIC
	ldx #$01
init_loop
	stx portb
	stx portb_bank_address
	inx
	inx
	cpx #$01
	bne init_loop
	
	ldy #0
	ldx #$ff
detect_loop
	stx portb
	cpx portb_bank_address
	bne no_bank
	txa
	sta portb_banks,y
	iny
no_bank
	dex
	dex
	cpx #$ff
	bne detect_loop
	sty portb_banks_count

	dec critic
	cli
	rts
	.endp

	.proc detect_vbxe		;OUT: X=1 for $d600, X=2 for $d700
	ldx #1
	lda $d600+vbxe_core_version_offset
	cmp #$10
	beq return
	inx
	lda $d700+vbxe_core_version_offset
	cmp #$10
	beq return
	ldx #0
return	stx vbxe_board
	rts
	.endp

	.proc display_error		;IN: <A>=<text, <X>=>text
	sta error_dlist.lms_adr
	stx error_dlist.lms_adr+1
	mwa #error_dlist sdlstl
	mva #$38 color0
	mva #$22 sdmctl
	mva #$e0 chbas
	jmp *

	.local error_dlist
:10	.byte $70
	.byte $46
lms_adr	.byte a($ffff),$70
	.byte $46,a(reset)
	.byte $41,a(error_dlist)
	.endl
	
	.local reset
	.sb "    PRESS RESET.    "
	.endl
	.endp

	.endp

	.proc next_block
wait	lda 20
old	cmp #0
	beq wait
	sta old+1

	inc block_number
	.endp

	.proc print_block
	lda block_number
	lsr
	lsr
	tay
	cpy #40
	bcs skip

	lda block_number
	and #3
	tax
	lda block_chars,x
	sta $ffff,y
sm_adr = *-2
skip	rts

	.local block_chars
	.byte $56,$59,$c2,$80
	.endl
	.endp

	.proc next_bank
	jsr next_block
	inc bank_number
	.endp

	.proc set_bank
	ldx bank_number
	lda portb_banks,x
	sta portb
	rts
	.endp

	.proc main
	mva #0 bank_number
	jsr set_bank

	ldx vbxe_board
error	beq error

	jsr copy_16k_bank
	jmp demo

	.proc copy_16k_bank
	mva #>portb_bank_address from+1

	ldy #>[.len d600_version+$ff]
	ldx #$00
loop
	lda $ff00,x
from	= *-2
	sta demo,x
to	= *-2
	inx
	bne loop
	inc from+1
	inc to+1
	dey
	bne loop
	rts
	.endp
	
	.endp

	.endp

	ini loader.init

	org portb_bank_address
	.local d600_version
	ins "ST-ONT-Blitter-$d600.xex",+6 ;$1800-$3fff, two 8k banks
	.endl
	ini loader.next_bank
	
	org portb_bank_address
	.local d700_version
	ins "ST-ONT-Blitter-$d700.xex",+6 ;$1800-$3fff, two 8k banks
	.endl
	ini loader.next_bank

	.macro m_segment
	.echo "Start of scene segment ",:1
	org portb_bank_address
	m_ins :1*$4000+$1000*0,$1000
	ini loader.next_block
	m_ins :1*$4000+$1000*1,$1000
	ini loader.next_block
	m_ins :1*$4000+$1000*2,$1000
	ini loader.next_block
	m_ins :1*$4000+$1000*3,$1000
	ini loader.next_bank
	.endm

	.rept scene_16k_banks
	m_segment #
	.endr

;	Last bank is not fully used
	org portb_bank_address
	.echo "Start of scene segment ",scene_16k_banks+1
	.byte $ff
;	m_ins :scene_16k_banks*$4000,scene_file_size-:scene_16k_banks*$4000
	ini loader.next_bank

	run loader.main
